@@ -99,7 +99,9 @@ class pypeMember(object):
             return representation
 
 class pypeScript(object):
-    
+
+    lastVisitedPath = '.'
+
     def __init__(self):
 
         self.BuiltinOptionTypes = ['int','str','float']
@@ -162,17 +164,33 @@ class pypeScript(object):
         self.OutputStream.write('\r')
         self.OutputStream.write('Progress: '+str(int(100 * self.Progress))+'%')
         self.OutputStream.flush()
+        self.InputInfo('Progress: '+str(int(100 * self.Progress))+'%')
  
     def EndProgress(self):
         self.OutputStream.write('\n')
-        
+ 
+    def InputInfo(self,prompt=''):
+        self.OutputText(prompt)
+        try:
+            self.InputStream.prompt(prompt)
+        except:
+            pass
+ 
     def InputText(self,prompt='',validator=None):
         self.OutputText(prompt)
-        text = self.InputStream.readline().rstrip('\n')
+        try:
+            self.InputStream.prompt(prompt)
+        except:
+            pass
+        text = self.InputStream.readline()
+        if text:
+            text = text.rstrip('\n')
         if validator:
             while not validator(text):
                 self.OutputText(prompt)
-                text = self.InputStream.readline().rstrip('\n')
+                text = self.InputStream.readline()
+                if text:
+                    text = text.rstrip('\n')
         return text
 
     def PrintMembers(self,members):
@@ -238,7 +256,7 @@ class pypeScript(object):
     def GetUsageString(self):
         usageString = ''
         scriptUsageString = os.path.splitext(self.ScriptName)[0]
-        if self.ScriptDoc != '':
+        if self.ScriptDoc:
             scriptUsageString += ' : ' + self.ScriptDoc
         useTextWrap = 1
         try:
@@ -292,6 +310,54 @@ class pypeScript(object):
         usageString += '\n'
         return usageString
 
+    def GetMarkdownUsageString(self):
+        usageString = ''
+        usageString += '---' + '\n'
+        usageString += 'layout: default' + '\n'
+        usageString += '---' + '\n'
+        usageString += '<h1>'
+        usageString += self.ScriptName
+        usageString += '</h1>'
+        usageString += '\n'
+        if self.ScriptDoc != '':
+            usageString += '<h2>Description</h2>' + '\n'
+            usageString += self.ScriptDoc + '\n'
+        for memberList in [self.InputMembers, self.OutputMembers]:
+            if memberList == self.InputMembers :
+                 usageString += '<h3>Input arguments</h3>' + '\n'
+            elif memberList == self.OutputMembers :
+                 usageString += '<h3>Output arguments</h3>' + '\n'
+            usageString += '<table class="vmtkscripts">' + '\n'
+            usageString += '<tr>' + '\n'
+            usageString += '<th>Argument</th><th>Variable</th><th>Type</th><th>Length</th><th>Range</th><th>Default</th><th>Description</th>\n'
+            usageString += '</tr>' + '\n'
+            for memberEntry in memberList:
+                memberUsageString = ''
+                memberName  = memberEntry.MemberName
+                option = memberEntry.OptionName
+                memberType = memberEntry.MemberType
+                memberLength = memberEntry.MemberLength
+                memberRange = memberEntry.MemberRange
+                memberDoc = memberEntry.MemberDoc
+                if option!='':
+                    default = 0
+                    if memberLength == 0:
+                        memberUsageString += '<td>' + option + '</td><td>' + memberName + '</td><td></td><td></td><td></td><td></td>'
+                    elif memberType in self.BuiltinOptionTypes + ["bool"]:
+                        default = self.__getattribute__(memberName)
+                        memberUsageString += '<td>' + option + '</td><td>' + memberName + '</td><td>' + memberType + '</td><td>' + str(memberLength) + '</td><td>' + str(memberRange) + '</td><td>' + str(default) + '</td>'
+                    else:
+                        memberUsageString += '<td>' + option + '</td><td>' + memberName + '</td><td>' + memberType + '</td><td>' + str(memberLength) + '</td><td></td><td></td>'
+                    if not memberDoc:
+                        memberDoc = ''
+                    memberUsageString += '<td>' + memberDoc + '</td>'
+                    #memberUsageString += ' | '
+                memberUsageString += '\n'
+                usageString += '<tr>' + memberUsageString + '</tr>' + '\n'
+            usageString += '</table>'
+        usageString += '\n'
+        return usageString
+
     def GetDokuWikiUsageString(self):
         usageString = ''
         usageString = '======'
@@ -382,6 +448,11 @@ class pypeScript(object):
                 self.OutputText(self.GetUsageString())
                 self.PrintLog('')
                 return 0
+            if arg == '--markdown':
+                self.PrintLog('')
+                self.OutputText(self.GetMarkdownUsageString())
+                self.PrintLog('')
+                return 0
             if arg == '--dokuwiki':
                 self.PrintLog('')
                 self.OutputText(self.GetDokuWikiUsageString())
