@@ -86,7 +86,7 @@ class vmtkNetworkEditor(pypes.pypeScript):
 
         self.CellIdsToMerge = []
 
-        self.OperationMode = 'add'
+        self.OperationMode = None
         self.PickMode = 'image'
 
         self.vmtkRenderer = None
@@ -159,188 +159,247 @@ class vmtkNetworkEditor(pypes.pypeScript):
             if self.PlaneWidgetZ:
                 self.PlaneWidgetZ.InteractionOff()
 
-    def KeyReleaseCallback(self,object,event):
+    def KeyReleaseCallback(self,obj,event):
         pass
         #key = object.GetKeySym()
         #if self.OperationMode == 'add':
         #    if key == 'Shift_L' or key == 'Shift_R':
         #        if self.PickMode == 'network':
         #            self.SetPickMode('image')
+    
+    def CheckMenu(self):
+        self.vmtkRenderer.RemoveKeyBinding('space')
+        self.vmtkRenderer.RemoveKeyBinding('c')
+        self.vmtkRenderer.RemoveKeyBinding('u')
+        self.vmtkRenderer.RemoveKeyBinding('+')
+        self.vmtkRenderer.RemoveKeyBinding('-')
+        self.vmtkRenderer.RemoveKeyBinding('=')
+        self.vmtkRenderer.RemoveKeyBinding('Return')
 
-    def KeyPressCallback(self,object,event):
-        key = object.GetKeySym()
-        operationModesDict = {'a':'add','d':'delete', 's':'split', 'm':'merge', 'l':'label'}
-        if key in operationModesDict.keys():
-            self.PrintLog('%s mode' % operationModesDict[key])
-            self.OperationMode = operationModesDict[key]
+    def ShowLabelCallback(self, obj):
+        self.ToggleLabels()
+
+    def AddCallback(self, obj):
+        self.InputInfo('Switched to add mode.\nCtrl + left click to add tubes.')
+        #self.PrintLog('Add mode')
+        self.OperationMode = 'add'
+        self.InitializeActiveSegment()
+        self.vmtkRenderer.AddKeyBinding('space','Toggle image/tube interaction',self.SpaceCallback, '2')
+        self.vmtkRenderer.AddKeyBinding('c','Cancel',self.CancelCallback,'2')
+        self.vmtkRenderer.AddKeyBinding('u','Undo',self.UndoCallback,'2')
+        self.vmtkRenderer.AddKeyBinding('+','Increase radius',self.PlusCallback,'2')
+        self.vmtkRenderer.AddKeyBinding('=','Increase radius',self.PlusCallback,'2')
+        self.vmtkRenderer.AddKeyBinding('-','Decrease radius',self.MinusCallback,'2')
+        self.vmtkRenderer.AddKeyBinding('Return','Accept tube',self.ReturnAddCallback,'2')
+        self.UpdateAndRender()
+
+    def DeleteCallback(self, obj):
+        self.InputInfo('Switched to delete mode.\nCtrl + left click to delete tubes.')
+        #self.PrintLog('Delete mode')
+        self.OperationMode = 'delete'
+        self.InitializeActiveSegment()
+        self.CheckMenu()
+        if self.PickMode == 'image':
+            self.SetPickMode('network')
             self.InitializeActiveSegment()
+        self.UpdateAndRender()
+
+    def SplitCallback(self, obj):
+        self.InputInfo('Switched to split mode.\nCtrl + left click to split tubes.')
+        #self.PrintLog('Split mode')
+        self.OperationMode = 'split'
+        self.InitializeActiveSegment()
+        self.CheckMenu()
+        if self.PickMode == 'image':
+            self.SetPickMode('network')
+            self.InitializeActiveSegment()
+        self.UpdateAndRender()
+
+    def LabelCallback(self, obj):
+        #self.PrintLog('Label mode')
+        self.InputInfo('Switched to label mode.\nCtrl + left click to add label.')
+        self.OperationMode = 'label'
+        self.InitializeActiveSegment()
+        self.CheckMenu()
+        if self.PickMode == 'image':
+            self.SetPickMode('network')
+            self.InitializeActiveSegment()
+        self.UpdateAndRender()
+
+    def MergeCallback(self, obj):
+        self.InputInfo('Switched to merge mode.\nCtrl + left click to select two tubes to merge.')
+        #self.PrintLog('Merge mode')
+        self.OperationMode = 'merge'
+        self.InitializeActiveSegment()
+        self.CheckMenu()
+        if self.PickMode == 'image':
+            self.SetPickMode('network')
+            self.InitializeActiveSegment()
+            self.CellIdsToMerge = []
+        self.vmtkRenderer.AddKeyBinding('Return','Accept merge',self.ReturnCallback,'2')
+        self.UpdateAndRender()
+
+    def SpaceCallback(self, obj):
+        self.TogglePickMode()
+        self.TogglePlaneWidget(self.PlaneWidgetX)
+        self.TogglePlaneWidget(self.PlaneWidgetY)
+        self.TogglePlaneWidget(self.PlaneWidgetZ)
+        self.InputInfo('Ctrl + left click to add seeds.')	
+
+    def CancelCallback(self, obj):
+        self.InitializeActiveSegment()
+
+    def UndoCallback(self, obj):
+        numberOfSeeds = self.ActiveSegmentSeedsPoints.GetNumberOfPoints()
+        if numberOfSeeds > 0:
+            self.ActiveSegmentSeedsPoints.SetNumberOfPoints(numberOfSeeds-1)
+            self.ActiveSegmentSeedsRadiusArray.SetNumberOfValues(numberOfSeeds-1)
+            self.ActiveSegmentSeeds.Modified()
+            self.AttachedCellIds[1] = -1
+            self.AttachedSubIds[1] = -1
+            self.AttachedPCoords[1] = 0.0
+        else:
+            self.AttachedCellIds[0] = -1
+            self.AttachedSubIds[0] = -1
+            self.AttachedPCoords[0] = 0.0
+        numberOfPoints = self.ActiveSegmentPoints.GetNumberOfPoints()
+        if numberOfPoints > 2:
+            self.ActiveSegmentPoints.SetNumberOfPoints(numberOfPoints-1)
+            self.ActiveSegmentRadiusArray.SetNumberOfValues(numberOfPoints-1)
+            self.ActiveSegmentCellArray.Initialize()
+            self.ActiveSegmentCellArray.InsertNextCell(numberOfPoints-1)
+            for i in range(numberOfPoints-1):
+                self.ActiveSegmentCellArray.InsertCellPoint(i)
+            self.ActiveSegment.Modified()
+        else:
+            self.ActiveSegmentPoints.SetNumberOfPoints(0)
+            self.ActiveSegmentCellArray.Initialize()
+            self.ActiveSegmentRadiusArray.SetNumberOfValues(0)
+            self.ActiveSegment.Modified()
+        self.UpdateAndRender()
+
+    def PlusCallback(self, obj):
+        numberOfSeeds = self.ActiveSegmentSeedsPoints.GetNumberOfPoints()
+        if numberOfSeeds > 0:
+            radius = self.ActiveSegmentSeedsRadiusArray.GetValue(numberOfSeeds-1)
+            radius += 0.1
+            self.ActiveSegmentSeedsRadiusArray.SetValue(numberOfSeeds-1,radius)
+            self.CurrentRadius = radius
+            self.ActiveSegmentSeeds.Modified()
+        numberOfPoints = self.ActiveSegmentPoints.GetNumberOfPoints()
+        if numberOfPoints > 0:
+            radius = self.ActiveSegmentRadiusArray.GetValue(numberOfPoints-1)
+            radius += 0.1
+            self.ActiveSegmentRadiusArray.SetValue(numberOfPoints-1,radius)
+            self.ActiveSegment.Modified()
+        self.UpdateAndRender()
+
+    def MinusCallback(self, obj):
+        numberOfSeeds = self.ActiveSegmentSeedsPoints.GetNumberOfPoints()
+        if numberOfSeeds > 0:
+            radius = self.ActiveSegmentSeedsRadiusArray.GetValue(numberOfSeeds-1)
+            radius -= 0.1
+            if radius > 0.0:
+                self.ActiveSegmentSeedsRadiusArray.SetValue(numberOfSeeds-1,radius)
+                self.CurrentRadius = radius
+                self.ActiveSegmentSeeds.Modified()
+        numberOfPoints = self.ActiveSegmentPoints.GetNumberOfPoints()
+        if numberOfPoints > 0:
+            radius = self.ActiveSegmentRadiusArray.GetValue(numberOfPoints-1)
+            radius -= 0.1
+            if radius > 0.0:
+                self.ActiveSegmentRadiusArray.SetValue(numberOfPoints-1,radius)
+                self.ActiveSegment.Modified()
+        self.UpdateAndRender()
+
+    def ReturnAddCallback(self, obj):
+        attachedPointId0 = -1
+        attachedPointId1 = -1
+        cellIdsToRemove = []
+        if self.AttachedCellIds[0] != -1:
+            attachedPointId0, hasSplit = self.SplitCellNoRemove(self.AttachedCellIds[0],self.AttachedSubIds[0],self.AttachedPCoords[0])
+            if hasSplit:
+                cellIdsToRemove.append(self.AttachedCellIds[0])
+        if self.AttachedCellIds[1] != -1:
+            attachedPointId1, hasSplit = self.SplitCellNoRemove(self.AttachedCellIds[1],self.AttachedSubIds[1],self.AttachedPCoords[1])
+            if hasSplit:
+                cellIdsToRemove.append(self.AttachedCellIds[1])
+        if cellIdsToRemove:
+            self.RemoveCells(cellIdsToRemove)
+        segment = self.ActiveSegmentActor.GetMapper().GetInput()
+        if self.UseActiveTubes:
+            self.RunActiveTube(segment)
+        segmentRadiusArray = segment.GetPointData().GetArray(self.RadiusArrayName)
+        numberOfSegmentPoints = segment.GetNumberOfPoints()
+        networkPoints = self.Network.GetPoints()
+        networkCellArray = self.Network.GetLines()
+        cellId = networkCellArray.InsertNextCell(numberOfSegmentPoints)
+        for i in range(numberOfSegmentPoints):
+            id = -1
+            if i == 0 and attachedPointId0 != -1:
+                id = attachedPointId0
+            elif i == numberOfSegmentPoints-1 and attachedPointId1 != -1:
+                id = attachedPointId1
+            else:
+                id = networkPoints.InsertNextPoint(segment.GetPoint(i))
+                self.NetworkRadiusArray.InsertTuple1(id,segmentRadiusArray.GetTuple1(i))
+            networkCellArray.InsertCellPoint(id)
+        self.NetworkLabelsArray.InsertValue(cellId,'')
+        self.Network.BuildCells()
+        self.Network.Modified()
+        self.NetworkTube.Modified()
+        self.UpdateLabels()
+        #if self.Image and self.PickMode == 'network':
+        #    self.SetPickMode('image')
+        self.InitializeSelection()
+        self.InitializeActiveSegment()
+
+    def ReturnCallback(self, obj):
+        numberOfActiveCells = self.ActiveSegment.GetNumberOfCells()
+        if numberOfActiveCells != 2 or len(self.CellIdsToMerge) != 2:
             return
-        if key == 'Tab':
-            self.ToggleLabels()
-        if self.OperationMode == 'add':
-            if key == 'space':
-                self.TogglePickMode()
-                self.TogglePlaneWidget(self.PlaneWidgetX)
-                self.TogglePlaneWidget(self.PlaneWidgetY)
-                self.TogglePlaneWidget(self.PlaneWidgetZ)
-            #elif key == 'Shift_L' or key == 'Shift_R':
-            #    if self.PickMode == 'image':
-            #        self.SetPickMode('network')
-            elif key == 'c':
-                self.InitializeActiveSegment()
-            elif key == 'u':
-                numberOfSeeds = self.ActiveSegmentSeedsPoints.GetNumberOfPoints()
-                if numberOfSeeds > 0:
-                    self.ActiveSegmentSeedsPoints.SetNumberOfPoints(numberOfSeeds-1)
-                    self.ActiveSegmentSeedsRadiusArray.SetNumberOfValues(numberOfSeeds-1)
-                    self.ActiveSegmentSeeds.Modified()
-                    self.AttachedCellIds[1] = -1
-                    self.AttachedSubIds[1] = -1
-                    self.AttachedPCoords[1] = 0.0
-                else:
-                    self.AttachedCellIds[0] = -1
-                    self.AttachedSubIds[0] = -1
-                    self.AttachedPCoords[0] = 0.0
-                numberOfPoints = self.ActiveSegmentPoints.GetNumberOfPoints()
-                if numberOfPoints > 2:
-                    self.ActiveSegmentPoints.SetNumberOfPoints(numberOfPoints-1)
-                    self.ActiveSegmentRadiusArray.SetNumberOfValues(numberOfPoints-1)
-                    self.ActiveSegmentCellArray.Initialize()
-                    self.ActiveSegmentCellArray.InsertNextCell(numberOfPoints-1)
-                    for i in range(numberOfPoints-1):
-                        self.ActiveSegmentCellArray.InsertCellPoint(i)
-                    self.ActiveSegment.Modified()
-                else:
-                    self.ActiveSegmentPoints.SetNumberOfPoints(0)
-                    self.ActiveSegmentCellArray.Initialize()
-                    self.ActiveSegmentRadiusArray.SetNumberOfValues(0)
-                    self.ActiveSegment.Modified()
-                self.Render()
-            elif key in ['equal','plus','minus']:
-                numberOfSeeds = self.ActiveSegmentSeedsPoints.GetNumberOfPoints()
-                if numberOfSeeds > 0:
-                    radius = self.ActiveSegmentSeedsRadiusArray.GetValue(numberOfSeeds-1)
-                    if key in ['equal','plus']:
-                        radius += 0.1
-                    else:
-                        radius -= 0.1
-                    self.ActiveSegmentSeedsRadiusArray.SetValue(numberOfSeeds-1,radius)
-                    self.CurrentRadius = radius
-                    self.ActiveSegmentSeeds.Modified()
-                numberOfPoints = self.ActiveSegmentPoints.GetNumberOfPoints()
-                if numberOfPoints > 0:
-                    radius = self.ActiveSegmentRadiusArray.GetValue(numberOfPoints-1)
-                    if key in ['equal','plus']:
-                        radius += 0.1
-                    else:
-                        radius -= 0.1
-                    self.ActiveSegmentRadiusArray.SetValue(numberOfPoints-1,radius)
-                    self.ActiveSegment.Modified()
-                self.Render()
-            elif key == 'Return':
-                attachedPointId0 = -1
-                attachedPointId1 = -1
-                cellIdsToRemove = []
-                if self.AttachedCellIds[0] != -1:
-                    attachedPointId0, hasSplit = self.SplitCellNoRemove(self.AttachedCellIds[0],self.AttachedSubIds[0],self.AttachedPCoords[0])
-                    if hasSplit:
-                        cellIdsToRemove.append(self.AttachedCellIds[0])
-                if self.AttachedCellIds[1] != -1:
-                    attachedPointId1, hasSplit = self.SplitCellNoRemove(self.AttachedCellIds[1],self.AttachedSubIds[1],self.AttachedPCoords[1])
-                    if hasSplit:
-                        cellIdsToRemove.append(self.AttachedCellIds[1])
-                if cellIdsToRemove:
-                    self.RemoveCells(cellIdsToRemove)
-                segment = self.ActiveSegmentActor.GetMapper().GetInput()
-                if self.UseActiveTubes:
-                    self.RunActiveTube(segment)
-                segmentRadiusArray = segment.GetPointData().GetArray(self.RadiusArrayName)
-                numberOfSegmentPoints = segment.GetNumberOfPoints()
-                networkPoints = self.Network.GetPoints()
-                networkCellArray = self.Network.GetLines()
-                cellId = networkCellArray.InsertNextCell(numberOfSegmentPoints)
-                for i in range(numberOfSegmentPoints):
-                    id = -1
-                    if i == 0 and attachedPointId0 != -1:
-                        id = attachedPointId0
-                    elif i == numberOfSegmentPoints-1 and attachedPointId1 != -1:
-                        id = attachedPointId1
-                    else:
-                        id = networkPoints.InsertNextPoint(segment.GetPoint(i))
-                        self.NetworkRadiusArray.InsertTuple1(id,segmentRadiusArray.GetTuple1(i))
-                    networkCellArray.InsertCellPoint(id)
-                self.NetworkLabelsArray.InsertValue(cellId,'a')
-                self.Network.BuildCells()
-                self.Network.Modified()
-                self.NetworkTube.Modified()
-                self.UpdateLabels()
-                #if self.Image and self.PickMode == 'network':
-                #    self.SetPickMode('image')
-                self.InitializeSelection()
-                self.InitializeActiveSegment()
-        elif self.OperationMode == 'delete':
-            if self.PickMode == 'image':
-                self.SetPickMode('network')
-                self.InitializeActiveSegment()
-        elif self.OperationMode == 'split':
-            if self.PickMode == 'image':
-                self.SetPickMode('network')
-                self.InitializeActiveSegment()
-        elif self.OperationMode == 'label':
-            if self.PickMode == 'image':
-                self.SetPickMode('network')
-                self.InitializeActiveSegment()
-        elif self.OperationMode == 'merge':
-            if self.PickMode == 'image':
-                self.SetPickMode('network')
-                self.InitializeActiveSegment()
-                self.CellIdsToMerge = []
-            if key == 'Return':
-                numberOfActiveCells = self.ActiveSegment.GetNumberOfCells()
-                if numberOfActiveCells != 2 or len(self.CellIdsToMerge) != 2:
-                    return
-                cell0ToMerge = self.Network.GetCell(self.CellIdsToMerge[0])
-                cell0PointIds = vtk.vtkIdList()
-                cell0PointIds.DeepCopy(cell0ToMerge.GetPointIds())
-                numberOfCell0Points = cell0PointIds.GetNumberOfIds()
-                cell1ToMerge = self.Network.GetCell(self.CellIdsToMerge[1])
-                cell1PointIds = vtk.vtkIdList()
-                cell1PointIds.DeepCopy(cell1ToMerge.GetPointIds())
-                numberOfCell1Points = cell1PointIds.GetNumberOfIds()
-                reverse = [False,False] 
-                if cell0PointIds.GetId(numberOfCell0Points-1) == cell1PointIds.GetId(0):
-                    reverse = [False,False]
-                elif cell0PointIds.GetId(numberOfCell0Points-1) == cell1PointIds.GetId(numberOfCell1Points-1):
-                    reverse = [False,True]
-                elif cell0PointIds.GetId(0) == cell1PointIds.GetId(numberOfCell1Points-1):
-                    reverse = [True,True]
-                elif cell0PointIds.GetId(0) == cell1PointIds.GetId(0):
-                    reverse = [True,False]
-                else:
-                    self.PrintLog('Error: trying to merge non-adjacent segments.')
-                    return
-                mergedLabel = self.NetworkLabelsArray.GetValue(self.CellIdsToMerge[0])
-                self.RemoveCells(self.CellIdsToMerge)
-                networkCellArray = self.Network.GetLines()
-                cellId = networkCellArray.InsertNextCell(numberOfCell0Points+numberOfCell1Points-1)
-                for i in range(numberOfCell0Points):
-                    loc = i
-                    if reverse[0]:
-                        loc = numberOfCell0Points - 1 - i
-                    networkCellArray.InsertCellPoint(cell0PointIds.GetId(loc))
-                for i in range(1,numberOfCell1Points):
-                    loc = i
-                    if reverse[1]:
-                        loc = numberOfCell1Points - 1 - i
-                    networkCellArray.InsertCellPoint(cell1PointIds.GetId(loc))
-                self.NetworkLabelsArray.InsertValue(cellId,mergedLabel)
-                self.Network.SetLines(networkCellArray)
-                self.Network.BuildCells()
-                self.Network.Modified()
-                self.NetworkTube.Modified()
-                self.UpdateLabels()
-                self.InitializeActiveSegment()
-                self.CellIdsToMerge = []
+        cell0ToMerge = self.Network.GetCell(self.CellIdsToMerge[0])
+        cell0PointIds = vtk.vtkIdList()
+        cell0PointIds.DeepCopy(cell0ToMerge.GetPointIds())
+        numberOfCell0Points = cell0PointIds.GetNumberOfIds()
+        cell1ToMerge = self.Network.GetCell(self.CellIdsToMerge[1])
+        cell1PointIds = vtk.vtkIdList()
+        cell1PointIds.DeepCopy(cell1ToMerge.GetPointIds())
+        numberOfCell1Points = cell1PointIds.GetNumberOfIds()
+        reverse = [False,False] 
+        if cell0PointIds.GetId(numberOfCell0Points-1) == cell1PointIds.GetId(0):
+            reverse = [False,False]
+        elif cell0PointIds.GetId(numberOfCell0Points-1) == cell1PointIds.GetId(numberOfCell1Points-1):
+            reverse = [False,True]
+        elif cell0PointIds.GetId(0) == cell1PointIds.GetId(numberOfCell1Points-1):
+            reverse = [True,True]
+        elif cell0PointIds.GetId(0) == cell1PointIds.GetId(0):
+            reverse = [True,False]
+        else:
+            self.PrintLog('Error: trying to merge non-adjacent segments.')
+            return
+        mergedLabel = self.NetworkLabelsArray.GetValue(self.CellIdsToMerge[0])
+        self.RemoveCells(self.CellIdsToMerge)
+        networkCellArray = self.Network.GetLines()
+        cellId = networkCellArray.InsertNextCell(numberOfCell0Points+numberOfCell1Points-1)
+        for i in range(numberOfCell0Points):
+            loc = i
+            if reverse[0]:
+                loc = numberOfCell0Points - 1 - i
+            networkCellArray.InsertCellPoint(cell0PointIds.GetId(loc))
+        for i in range(1,numberOfCell1Points):
+            loc = i
+            if reverse[1]:
+                loc = numberOfCell1Points - 1 - i
+            networkCellArray.InsertCellPoint(cell1PointIds.GetId(loc))
+        self.NetworkLabelsArray.InsertValue(cellId,mergedLabel)
+        self.Network.SetLines(networkCellArray)
+        self.Network.BuildCells()
+        self.Network.Modified()
+        self.NetworkTube.Modified()
+        self.UpdateLabels()
+        self.InitializeActiveSegment()
+        self.CellIdsToMerge = []
 
     def RemoveCell(self,cellId):
         self.RemoveCells([cellId])
@@ -393,7 +452,7 @@ class vmtkNetworkEditor(pypes.pypeScript):
         self.Network.BuildCells()
         return splitPointId, True
 
-    def LeftButtonPressCallback(self,object,event):
+    def LeftButtonPressCallback(self,obj,event):
         if self.PickMode != 'network':
             return
         if self.vmtkRenderer.RenderWindowInteractor.GetControlKey() == 0:
@@ -455,14 +514,11 @@ class vmtkNetworkEditor(pypes.pypeScript):
             self.NetworkTube.Modified()
             self.UpdateLabels()
         elif self.OperationMode == 'label':
+            self.LabelCellId = cellId
             label = self.NetworkLabelsArray.GetValue(cellId)
             if label:
                 self.PrintLog("Current label: %s" % label)
-            newLabel = self.InputText("Please input new label: ")
-            self.NetworkLabelsArray.SetValue(cellId,newLabel)
-            self.InitializeSelection()
-            self.Network.Modified()
-            self.UpdateLabels()
+            self.vmtkRenderer.PromptAsync("Please input new label: ",self.QueryLabelCallback)
         elif self.OperationMode == 'merge':
             if self.ActiveSegment.GetNumberOfCells() > 1:
                 self.InitializeActiveSegment()
@@ -484,6 +540,13 @@ class vmtkNetworkEditor(pypes.pypeScript):
             self.NetworkTube.Modified()
             self.Render()
 
+    def QueryLabelCallback(self,newLabel):
+        self.NetworkLabelsArray.SetValue(self.LabelCellId,newLabel)
+        self.LabelCellId = None
+        self.InitializeSelection()
+        self.Network.Modified()
+        self.UpdateLabels()
+
     def InitializeSelection(self):
         self.SelectionPoints.SetNumberOfPoints(0)
         self.SelectionRadiusArray.SetNumberOfValues(0)
@@ -507,7 +570,7 @@ class vmtkNetworkEditor(pypes.pypeScript):
         self.AttachedPCoords = [0.0,0.0]
         self.Render()
 
-    def MouseMoveCallback(self,object,event):
+    def MouseMoveCallback(self,obj,event):
         if self.PickMode != 'network':
             return
         if self.vmtkRenderer.RenderWindowInteractor.GetControlKey() == 0:
@@ -516,7 +579,7 @@ class vmtkNetworkEditor(pypes.pypeScript):
             return
         if self.Network.GetNumberOfCells() == 0:
             return
-        eventPosition = object.GetEventPosition()
+        eventPosition = self.vmtkRenderer.RenderWindowInteractor.GetEventPosition()
         result = self.CellPicker.Pick(float(eventPosition[0]),float(eventPosition[1]),0.0,self.vmtkRenderer.Renderer)
         if result == 0:
             return
@@ -545,12 +608,14 @@ class vmtkNetworkEditor(pypes.pypeScript):
         self.Selection.Modified()
         self.Render()
 
-    def PlaneStartInteractionCallback(self,object,event):
+    def PlaneStartInteractionCallback(self,obj,event):
         if self.PickMode != 'image':
             return
+        if self.OperationMode != 'add':
+            return
         if self.vmtkRenderer.RenderWindowInteractor.GetControlKey() == 0:
             return
-        point = object.GetCurrentCursorPosition()
+        point = obj.GetCurrentCursorPosition()
         radius = self.CurrentRadius
         if self.ActiveSegmentSeeds.GetNumberOfPoints() == 0:
             self.ActiveSegmentSeedsPoints.InsertNextPoint(point)
@@ -582,6 +647,9 @@ class vmtkNetworkEditor(pypes.pypeScript):
     def Render(self):
         self.vmtkRenderer.RenderWindow.Render()
 
+    def UpdateAndRender(self):
+        self.vmtkRenderer.Render(interactive=0)
+
     def RunActiveTube(self,segment):
         activeTubes = vmtkactivetubes.vmtkActiveTubes()
         activeTubes.Centerline = segment
@@ -610,6 +678,9 @@ class vmtkNetworkEditor(pypes.pypeScript):
             self.vmtkRenderer.Initialize()
             self.OwnRenderer = 1
 
+        self.vmtkRenderer.ExitAfterTextInputMode = False
+        self.vmtkRenderer.RegisterScript(self) 
+
         if self.Image and (not self.PlaneWidgetX or not self.PlaneWidgetY or not self.PlaneWidgetZ):
             imageViewer = vmtkimageviewer.vmtkImageViewer()
             imageViewer.Image = self.Image
@@ -760,7 +831,7 @@ class vmtkNetworkEditor(pypes.pypeScript):
             self.NetworkLabelsArray.SetName(self.LabelsArrayName)
             self.NetworkLabelsArray.SetNumberOfValues(self.Network.GetNumberOfCells())
             for i in range(self.Network.GetNumberOfCells()):
-                self.NetworkLabelsArray.SetValue(i,'a')
+                self.NetworkLabelsArray.SetValue(i,'')
             self.Network.GetCellData().AddArray(self.NetworkLabelsArray)
         self.CellCenters = vtk.vtkCellCenters()
         self.CellCenters.SetInput(self.Network)
@@ -787,7 +858,12 @@ class vmtkNetworkEditor(pypes.pypeScript):
         self.CellPicker.AddPickList(self.NetworkActor)
         self.CellPicker.PickFromListOn()
 
-        self.vmtkRenderer.RenderWindowInteractor.AddObserver("KeyPressEvent", self.KeyPressCallback)
+        self.vmtkRenderer.AddKeyBinding('a','Add mode.',self.AddCallback)
+        self.vmtkRenderer.AddKeyBinding('d','Delete mode.',self.DeleteCallback)
+        self.vmtkRenderer.AddKeyBinding('m','Merge mode.',self.MergeCallback)
+        self.vmtkRenderer.AddKeyBinding('s','Split mode.',self.SplitCallback)
+        self.vmtkRenderer.AddKeyBinding('l','Label mode.',self.LabelCallback)
+        self.vmtkRenderer.AddKeyBinding('Tab','Show labels.',self.ShowLabelCallback)
         self.vmtkRenderer.RenderWindowInteractor.AddObserver("KeyReleaseEvent", self.KeyReleaseCallback)
         self.vmtkRenderer.RenderWindowInteractor.AddObserver("LeftButtonPressEvent", self.LeftButtonPressCallback)
         self.vmtkRenderer.RenderWindowInteractor.AddObserver("MouseMoveEvent", self.MouseMoveCallback)
@@ -802,8 +878,6 @@ class vmtkNetworkEditor(pypes.pypeScript):
             self.PlaneWidgetZ.UseContinuousCursorOn()
             self.PlaneWidgetZ.AddObserver("StartInteractionEvent", self.PlaneStartInteractionCallback)
 
-        self.PickMode = 'image'
-
         self.FirstRender()
 
         self.Surface = self.NetworkTube.GetOutput()
