@@ -27,7 +27,7 @@
 #undef itkExceptionMacro  
 #define itkExceptionMacro(x) \
   { \
-  ::itk::OStringStream message; \
+  std::ostringstream message; \
   message << "itk::ERROR: " << this->GetNameOfClass() \
           << "(" << this << "): " x; \
   std::cout << message.str().c_str() << std::endl; \
@@ -36,7 +36,7 @@
 #undef itkGenericExceptionMacro  
 #define itkGenericExceptionMacro(x) \
   { \
-  ::itk::OStringStream message; \
+  std::ostringstream message; \
   message << "itk::ERROR: " x; \
   std::cout << message.str().c_str() << std::endl; \
   }
@@ -49,10 +49,10 @@
 // reading directions from the file.
 // The GE5 reader was fixed just after the itk 3.2 release
 //
-#if (ITK_VERSION_MAJOR > 3) || \
-((ITK_VERSION_MAJOR == 3 && ITK_VERSION_MINOR >= 2))
-#define USE_ITKGE5READER
-#endif
+//#if (ITK_VERSION_MAJOR > 3) || \
+//((ITK_VERSION_MAJOR == 3 && ITK_VERSION_MINOR >= 2))
+//#define USE_ITKGE5READER
+//#endif
 
 #ifdef USE_ITKGE5READER
 #include "itkImageIOFactory.h"
@@ -75,8 +75,8 @@
 #include "itkMetaImageIO.h"
 #include "itkNrrdImageIO.h"
 #include "itkGE5ImageIO.h"
-#include "itkBrains2MaskImageIOFactory.h"
-#include "itkBrains2MaskImageIO.h"
+//#include "itkBrains2MaskImageIOFactory.h"
+//#include "itkBrains2MaskImageIO.h"
 #include "itkNiftiImageIO.h"
 #include "itkVTKImageIO.h"
 #include "itkTIFFImageIO.h"
